<?php
/**
 * Classe para construo de relatrios tabulares simples
 * @author Pablo Dall'Oglio
 */
class TSimpleReport
{
    private $database;
    private $sql;
    private $writter;
    private $columns;
    private $group;
    private $total;
    
    /**
     * Mtodo construtor
     */
    public function __construct()
    {
        $this->columns = array();
    }
    
    /**
     * Define o banco de dados para a consulta SQL
     * @param $database nome do banco de dados
     */
    public function setDatabase($database)
    {
        $this->database = $database;
    }
    
    /**
     * Define a consulta SQL para gerao do relatrio
     * @param $sql consulta em SQL
     */
    public function setQuery($sql)
    {
        $this->sql = $sql;
    }
    
    /**
     * Define um objeto "escritor" do relatrio
     * @param $writter objeto que implementa a interface ITableWritter
     */
    public function setReportWritter($writter)
    {
        $this->writter = $writter;
    }
    
    /**
     * Adiciona uma coluna ao relatrio
     * @param $alias nome da coluna na consulta SQL
     * @param $label rtulo do campo (ttulo da coluna)
     * @param $align alinhamento da coluna
     */
    public function addColumn($alias, $label, $align)
    {
        $this->columns[] = array('alias'=>$alias, 'label'=>$label, 'align'=>$align);
    }
    
    /**
     * Define uma quebra no relatrio
     * @param $alias nome da coluna de quebra na consulta SQL
     */
    public function setGroup($alias)
    {
        $this->group = $alias;
    }
    
    /**
     * Cria um totalizador no relatrio
     * @param $alias   nome da coluna a ser totalizada
     * @param $formula frmula a ser aplicada
     */
    public function setTotal($alias, $formula)
    {
        $this->total[$alias] = $formula;
        $this->results[$alias] = 0;
    }
    
    /**
     * Gera o relatrio
     */
    public function generate()
    {
        if (!isset($this->database)) // verifica se foi definido o banco de dados
        {
            throw new Exception('Banco de dados no definido');
        }
        if (!isset($this->sql)) // verifica se foi definida a consulta SQL
        {
            throw new Exception('Consulta SQL no definida');
        }
        if (!isset($this->writter)) // verifica se foi definido o escritor de tabelas
        {
            throw new Exception('Escritor de tabelas no definido');
        }
        if (count($this->columns) == 0) // verifica se foram definidas as colunas do relatrio
        {
            throw new Exception('Colunas do relatrio no definidas');
        }
        
        // adiciona uma linha para o cabealho (ttulos das colunas)
        $this->writter->addRow();
        foreach ($this->columns as $column)
        {
            // adiciona as colunas de cabealho
            $this->writter->addCell($column['label'], $column['align'], 'title');
        }
        
        $conn = TConnection::open($this->database); // abre conexo com a base de dados
        $result = $conn->query($this->sql); // executa a instruo SQL
        
        // varivel para controle de troca de grupo
        $last_group = NULL;
        
        $colore = FALSE;
        // percorre os resultados
        foreach ($result as $row)
        {
            // define o estilo a ser utilizado
            $style = $colore ? 'datai' : 'datap';
            
            // verifica se o relatrio tem agrupamento
            if (isset($this->group))
            {
                // verifica se a coluna de agrupamento trocou de valor
                if ($last_group !== $row[$this->group])
                {
                    // totalizao na troca de grupo
                    if (isset($this->total) AND $last_group !== NULL)
                    {
                        $this->printTotals();
                    }
                    
                    // acrescenta uma linha para o grupo
                    $this->writter->addRow();
                    
                    // exibe a clula de agrupamento
                    $this->writter->addCell($row[$this->group], 'left', 'group');
                    
                    // completa a linha com as demais clulas vazias
                    for ($n = 1; $n <= count($this->columns)-1; $n++)
                    {
                        $this->writter->addCell('', 'left', 'group');
                    }
                    
                    // atualiza a varivel de controle para troca de grupo
                    $last_group = $row[$this->group];
                }
            }
            
            // adiciona uma linha para os dados
            $this->writter->addRow();
            
            // adiciona as colunas com os dados
            foreach ($this->columns as $column)
            {
                $alias   = $column['alias'];
                $content = $row[$alias];
                $this->writter->addCell($content, $column['align'], $style);
                
                // realiza as totalizaes da coluna se necessrio
                if (isset($this->total[$alias]))
                {
                    $formula = $this->total[$alias];
                    if ($formula == 'count')
                    {
                        $this->results[$alias] ++;
                    }
                    else if ($formula == 'sum')
                    {
                        $this->results[$alias] += $content;
                    }
                }
            }
            
            // alterna varivel de controle para cor de fundo
            $colore = !$colore;
        }
        
        // totalizao final
        if (isset($this->total))
        {
            $this->printTotals();
        }
    }
    
    /**
     * Exibe os totais das frmulas
     */
    private function printTotals()
    {
        // adiciona uma linha
        $this->writter->addRow();
        
        // percorre as colunas
        foreach ($this->columns as $column)
        {
            $alias = $column['alias'];
            
            // verifica se h totalizao para a coluna
            if (isset($this->results[$alias]))
            {
                // exibe a totalizao da coluna
                $formula = $this->total[$alias];
                $this->writter->addCell($formula . ': '. $this->results[$alias],
                                        $column['align'], 'total');
                                        
                // reinicializa os totais da coluna
                $this->results[$alias] = 0;
            }
            else
            {
                // exibe uma clula vazia
                $this->writter->addCell('', $column['align'], 'total');
            }
        }
    }
    
    /**
     * Armazena o relatrio em um arquivo
     * @param $filename localizao do arquivo
     */
    public function save($filename)
    {
        $this->writter->save($filename);
    }
}
?>